<?php
/**
* Plugin Name: AVIF / WebP Images
* Plugin URI: https://remydcf.dev/avif-webp-images
* Description: This plugin rewrites image sources with AVIF and WebP files if found in the same folder (and if the file format is marked as accepted by the HTTP Accept header).
* Version: 1
* Author: Rémy Da Costa Faro
* Author URI: https://remydcf.dev
* */

function endsWith($haystack, $needle) {
    $length = strlen( $needle );
    if( !$length ) {
        return true;
    }
    return substr( $haystack, -$length ) === $needle;
}

function replace_file_extension($filename, $extension) {
    if (endsWith($filename, ".png")) {
        return rtrim($filename, "png") . $extension;
    } elseif (endsWith($filename, ".jpg")) {
        return rtrim($filename, "jpg") . $extension;
    } elseif (endsWith($filename, ".jpeg")) {
        return rtrim($filename, "jpeg") . $extension;
    }

    return $filename;
}

function replace_file_ext_with_type_if_exists($match, $extension) {
    $filename = wp_upload_dir()["basedir"] . "/" . $match["link"];

    $newtype_filename = replace_file_extension($filename, $extension);
    if (file_exists($newtype_filename)) {
        return str_replace($match["link"], replace_file_extension($match["link"], $extension), $match[0]);
    }

    $newtype_filename = $filename . "." . $extension;
    if (file_exists($newtype_filename)) {
        return str_replace($match["link"], $match["link"] . "." . $extension, $match[0]);
    }

    return null;
}

function reroute_image_file($match) {
    if (strpos($_SERVER['HTTP_ACCEPT'], "image/avif") !== false) {
        $replaced_match = replace_file_ext_with_type_if_exists($match, "avif");

        if (!empty($replaced_match)) {
            return $replaced_match;
        }
    }

    if (strpos($_SERVER['HTTP_ACCEPT'], "image/webp") !== false) {
        $replaced_match = replace_file_ext_with_type_if_exists($match, "webp");

        if (!empty($replaced_match)) {
            return $replaced_match;
        }
    }

    return $match[0];
}

function filter_content_for_avif_webp($content) {
    global $post;

    if ( is_singular() && in_the_loop() && is_main_query() ) {
        $escaped_site_url = preg_quote(content_url());
        $escaped_site_url = str_replace("/", "\/", $escaped_site_url);

        $regex = "/" . $escaped_site_url . "\/uploads\/(?P<link>.*\.(jpg|png|jpeg))/mUi";

        $content = preg_replace_callback($regex, "reroute_image_file", $content);
    }
    
    return $content;
}

add_filter('the_content', 'filter_content_for_avif_webp', 9999);

require plugin_dir_path( __FILE__ ) . '/plugin-update-checker/plugin-update-checker.php';

$updateChecker = Puc_v4_Factory::buildUpdateChecker(
    'https://remydcf.dev/avif-webp-images/info.json',
    __FILE__,
    'avif-webp-images'
);